@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2023 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -37,7 +37,15 @@ Channel *ThreadModule::newChannel()
 
 Channel *ThreadModule::getChannel(const std::string &name)
 {
-	return Channel::getChannel(name);
+	Lock lock(namedChannelMutex);
+
+	auto it = namedChannels.find(name);
+	if (it != namedChannels.end())
+		return it->second;
+
+	Channel *c = new Channel();
+	namedChannels[name].set(c, Acquire::NORETAIN);
+	return c;
 }
 
 const char *ThreadModule::getName() const
